import josx.platform.rcx.*;

public class MiniTank implements SensorConstants {
   
   private static int repeating = 0;
   
   public static void main(String [] args) {
      byte actions = 9;
      byte states = 24;
      QUtil qutil = new QUtil(actions, states);
      QBrain q = new QBrain(actions, states, qutil);
      byte[] e = new byte[5]; // The environment values
      byte [] motorCommand = new byte[2]; // Commands to send to motors
      
      Sensor.S2.setTypeAndMode (SENSOR_TYPE_LIGHT, SENSOR_MODE_PCT);
      Sensor.S2.activate();
      
      while(true) {
         getEnvironmentVals(e);
                  
         byte a = q.getAction(e);
         
         qutil.getCommands(a, motorCommand);
         
         performAction(motorCommand); 
      }
   }
   
   public static void performAction(byte [] motorCommand) {
      // Motor A:
      if(motorCommand[0] == 0)
         Motor.A.stop();
      else if(motorCommand[0] == 1)
         Motor.A.forward();
      else
         Motor.A.backward();
         
      // Motor C:
      if(motorCommand[1] == 0)
         Motor.C.stop();
      else if(motorCommand[1] == 1)
         Motor.C.forward();
      else
         Motor.C.backward();
      try {
         Thread.sleep(450);
      } catch(InterruptedException ie) {}
      Motor.A.stop();
      Motor.C.stop();
   }
   
   public static void getEnvironmentVals(byte [] e) {
      // Collect environment percepts:
      e[0] = compareLight();
      
      if(Sensor.S1.readBooleanValue()) {
         e[1] = 1;
         Sound.beep();
      } else
         e[1] = 0;
         
      if(Sensor.S3.readBooleanValue()) {
         e[2] = 1;
         Sound.beep();
      } else
         e[2] = 0;
         
      // Internal state - if light value has remained same, indicate this:
      int previousLightValue = e[4];
      int lightValue = Sensor.S2.readValue();
      if(previousLightValue == lightValue) {
         ++repeating;
      } else
         repeating = 0;
      if(repeating > 5)
         e[3] = 1;
      else
         e[3] = 0;
      
      
      e[4] = (byte)Sensor.S2.readValue();
      LCD.showNumber(e[4]);
   }
   
   /**
    * Simulate 2 light sensors by rotating left then right.
    */
   public static byte compareLight() {
      
      // Move left:
      Motor.C.forward();
      Motor.A.backward();
      try {
            Thread.sleep(250);
      } catch(InterruptedException ie) {}
      Motor.A.stop();
      Motor.C.stop();
      byte left = (byte)Sensor.S2.readValue();
      
      
      // Move right:
      Motor.A.forward();
      Motor.C.backward();
      try {
            Thread.sleep(500);
      } catch(InterruptedException ie) {}
      Motor.A.stop();
      Motor.C.stop();
      byte right = (byte)Sensor.S2.readValue();   
      
      // Move back:
      Motor.C.forward();
      Motor.A.backward();
      try {
            Thread.sleep(250);
      } catch(InterruptedException ie) {}
      Motor.A.stop();
      Motor.C.stop();
      
      // Convert readings to byte value:
      if(left > right)
         return 0;
      else if(left < right)
         return 1;
      else
         return 2;
   }
}